use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
APIS
Alders
Andreas
Ayachit
CPAN
Choroba
Constants
DROLSKY
DROLSKY's
Dave
File
Fitzner
Hendricks
LibMagic
Michael
Mithun
OO
Olaf
Paul
PayPal
Rolsky
Rolsky's
Tom
Wise
Wyant
and
ascii
autarch
choroba
drolsky
featureful
freedesktop
homebrew
lib
libmagic
mayachit
michael
olaf
pabs3
uncompress
wyant
